searchNodes=[{"ref":"Dawdle.TestHelper.html","title":"Dawdle.TestHelper","type":"module","doc":"Helper functions for writing tests that interact with Dawdle handlers."},{"ref":"Dawdle.TestHelper.html#clear_all_handlers/0","title":"Dawdle.TestHelper.clear_all_handlers/0","type":"function","doc":"Clear all currently registered Dawdle handlers."},{"ref":"Dawdle.html","title":"Dawdle","type":"module","doc":"API for the Dawdle messaging system."},{"ref":"Dawdle.html#call/1","title":"Dawdle.call/1","type":"function","doc":"Send a function to be executed. Note that this is an experimental API. The function is encoded and enqueued and will be executed on a node running the Dawdle listener. Even if the listener is running on the current node, the function may still be executed on another node. The passed function is evaluated for its side effects and any return value is ignored. Returns :ok when the function is successfully enqueued. Otherwise, returns an error tuple. Examples iex&gt; Dawdle.call(fn -&gt; ...&gt; # Do something expensive... ...&gt; :ok ...&gt; end) :ok"},{"ref":"Dawdle.html#call_after/2","title":"Dawdle.call_after/2","type":"function","doc":"Send a function to be executed after a delay. Note that this is an experimental API. The function is encoded and enqueued and will be executed on a node running the Dawdle listener after the specified delay. Even if the listener is running on the current node, the function may still be executed on another node. The passed function is evaluated for its side effects and any return value is ignored. Returns :ok when the function is successfully enqueued. Otherwise, returns an error tuple. Examples iex&gt; Dawdle.call_after(5, fn -&gt; ...&gt; # Do something later... ...&gt; :ok ...&gt; end) :ok"},{"ref":"Dawdle.html#call_after/3","title":"Dawdle.call_after/3","type":"function","doc":"(DEPRECATED) Set a callback to be called the eafter delay ms Examples iex&gt; Dawdle.call_after(fn _arg -&gt; ...&gt; # Do something later... ...&gt; :ok ...&gt; end, 1, 5) :ok"},{"ref":"Dawdle.html#handler_count/0","title":"Dawdle.handler_count/0","type":"function","doc":"Returns the total number of subscribers."},{"ref":"Dawdle.html#handler_count/1","title":"Dawdle.handler_count/1","type":"function","doc":"Returns the number of subscribers to a specific event."},{"ref":"Dawdle.html#register_all_handlers/0","title":"Dawdle.register_all_handlers/0","type":"function","doc":"Registers all known event handlers. Dawdle searches through all loaded modules for any that implement the Dawdle.Handler behaviour and registers them. This is automatically called when the :dawdle application starts."},{"ref":"Dawdle.html#register_handler/2","title":"Dawdle.register_handler/2","type":"function","doc":"Registers an event handler. After calling this function, the next time the specified event occurs, then the handler function will be called with data from that event."},{"ref":"Dawdle.html#signal/2","title":"Dawdle.signal/2","type":"function","doc":"Signals an event. The event is encoded and enqueued and will be processed by a handler running on a node running the Dawdle listener. See Dawdle.Handler for information on creating event handlers. Use the :delay option to delay the signaling of the event. Returns :ok when the event is successfully enqueued. Otherwise, returns an error tuple. Examples defmodule MyApp.TestEvent do defstruct :foo, :bar end Dawdle.signal(%MyApp.TestEvent{foo: 1, bar: 2}) Dawdle.signal(%MyApp.TestEvent{foo: 1, bar: 2}, delay: 5)"},{"ref":"Dawdle.html#start_pollers/0","title":"Dawdle.start_pollers/0","type":"function","doc":"Starts the pollers if they were not started automatically at application startup."},{"ref":"Dawdle.html#stop_pollers/0","title":"Dawdle.stop_pollers/0","type":"function","doc":"Stops any running pollers."},{"ref":"Dawdle.html#unregister_handler/1","title":"Dawdle.unregister_handler/1","type":"function","doc":"Unregisters an event handler."},{"ref":"Dawdle.html#t:argument/0","title":"Dawdle.argument/0","type":"type","doc":""},{"ref":"Dawdle.html#t:callback/0","title":"Dawdle.callback/0","type":"type","doc":""},{"ref":"Dawdle.html#t:duration/0","title":"Dawdle.duration/0","type":"type","doc":""},{"ref":"Dawdle.html#t:event/0","title":"Dawdle.event/0","type":"type","doc":""},{"ref":"Dawdle.html#t:handler/0","title":"Dawdle.handler/0","type":"type","doc":""},{"ref":"Dawdle.Handler.html","title":"Dawdle.Handler","type":"behaviour","doc":"Defines a handler for queued events. To define an event handler, use Dawdle.Handler and provide a list of event types (structs) that you wish to handle. Then, override the callback handle_event/1. Examples defmodule MyApp.TestEvent do defstruct :foo, :bar end defmodule MyApp.TestEventHandler do use Dawdle.Handler, only: [MyApp.TestEvent] alias MyApp.TestEvent def handle_event(%TestEvent{foo: 1}) do # Do something... end def handle_event(%TestEvent{bar: 2}) do # Do something else... end def handle_event(%TestEvent{}) do # Default case end end"},{"ref":"Dawdle.Handler.html#c:handle_event/1","title":"Dawdle.Handler.handle_event/1","type":"callback","doc":"This function is called when Dawdle pulls an event of the appropriate type from the queue. It will only be called with events specified in the use Dawdle.Handler statement. The function is executed for its side effects and the return value is ignored."},{"ref":"Dawdle.Backend.html","title":"Dawdle.Backend","type":"behaviour","doc":"Behaviour for Dawdle backends. Dawdle backends are responsible for interfacing with the actual message queue."},{"ref":"Dawdle.Backend.html#c:delete/2","title":"Dawdle.Backend.delete/2","type":"callback","doc":""},{"ref":"Dawdle.Backend.html#c:init/0","title":"Dawdle.Backend.init/0","type":"callback","doc":""},{"ref":"Dawdle.Backend.html#new/0","title":"Dawdle.Backend.new/0","type":"function","doc":"Returns an initialized backend. Looks up the preferred backend in the application environment and calls the backend&#39;s init/0 callback."},{"ref":"Dawdle.Backend.html#c:queues/0","title":"Dawdle.Backend.queues/0","type":"callback","doc":""},{"ref":"Dawdle.Backend.html#c:recv/1","title":"Dawdle.Backend.recv/1","type":"callback","doc":""},{"ref":"Dawdle.Backend.html#c:send/1","title":"Dawdle.Backend.send/1","type":"callback","doc":""},{"ref":"Dawdle.Backend.html#c:send_after/2","title":"Dawdle.Backend.send_after/2","type":"callback","doc":""},{"ref":"Dawdle.Backend.html#t:delay_secs/0","title":"Dawdle.Backend.delay_secs/0","type":"type","doc":""},{"ref":"Dawdle.Backend.html#t:queue/0","title":"Dawdle.Backend.queue/0","type":"type","doc":""},{"ref":"Dawdle.Backend.html#t:recv_message/0","title":"Dawdle.Backend.recv_message/0","type":"type","doc":""},{"ref":"Dawdle.Backend.html#t:send_message/0","title":"Dawdle.Backend.send_message/0","type":"type","doc":""},{"ref":"Dawdle.Backend.Local.html","title":"Dawdle.Backend.Local","type":"module","doc":"Implementation of the Dawdle.Backend behaviour that queues all events locally on a single node. This is intended for use in testing and development where a &quot;live&quot; backend like SQS is not available or desirable."},{"ref":"Dawdle.Backend.Local.html#child_spec/1","title":"Dawdle.Backend.Local.child_spec/1","type":"function","doc":"Returns a specification to start this module under a supervisor. See Supervisor."},{"ref":"Dawdle.Backend.SQS.html","title":"Dawdle.Backend.SQS","type":"module","doc":"Implementation of the Dawdle.Backend behaviour for Amazon SQS."},{"ref":"Dawdle.MessageEncoder.html","title":"Dawdle.MessageEncoder","type":"behaviour","doc":"Behaviour for Dawdle event encoding. Dawdle wants to enqueue Elixir structs and other Erlang terms. This behaviour specifies an interface for translating an event, i.e., term, into a string that can be safely enqueued."},{"ref":"Dawdle.MessageEncoder.html#c:decode/1","title":"Dawdle.MessageEncoder.decode/1","type":"callback","doc":"Decode a string pulled from the queue into its original representation."},{"ref":"Dawdle.MessageEncoder.html#c:encode/1","title":"Dawdle.MessageEncoder.encode/1","type":"callback","doc":"Encode an event into a string that is safe to enqueue."},{"ref":"Dawdle.MessageEncoder.Term.html","title":"Dawdle.MessageEncoder.Term","type":"module","doc":"Implements the Dawdle.MessageEncoder behaviour using :erlang.term_to_binary/1 and :erlang.binary_to_term/1. The results are encoded with Base64 to ensure that they are safe to enqueue."},{"ref":"readme.html","title":"Dawdle","type":"extras","doc":"Dawdle Dawdle weaponizes Amazon SQS for use in your Elixir applications. Use it when you want to handle something later, or, better yet, when you want someone else to handle it. Put simply, you can use Dawdle to signal an event. That event can then be handled by another process within the current Erlang node, or another node altogether. You can even signal events with a delay, similar to Process.send_after/4, but in a distributed and node-failure-tolerant manner. The events are encoded and enqueued into Amazon AWS&#39;s Simple Queue Service (SQS). This means that if, for example, you&#39;re running your BEAM system in a Kubernetes cluster, and one or more of your pods die or are restarted or terminated or what have you, your events will still fire and be handled by whatever pods are available. If no pods are available for some reason, the events will still be preserved and can be handled when a pod is available again to service them."},{"ref":"readme.html#installation","title":"Dawdle - Installation","type":"extras","doc":"Add dawdle to your list of dependencies in mix.exs: def deps do [ {:dawdle, &quot;~&gt; 0.5.1&quot;} ] end"},{"ref":"readme.html#configuration","title":"Dawdle - Configuration","type":"extras","doc":"Dawdle provides two backends: A local one for development and testing which does not require access to SQS (the default), and an SQS one for use in an AWS environment. To enable the SQS backend, set the following in your application&#39;s config.exs: config :dawdle, backend: Dawdle.Backend.SQS To disable the Dawdle queue listener on a node, use the following: config :dawdle, start_listener: false You can also set the environment variables DAWDLE_BACKEND or DAWDLE_START_LISTENER. To configure your SQS queues, set the following: config :dawdle, Dawdle.Backend.SQS, region: &quot;us-east-1&quot;, delay_queue: &quot;my-dawdle-delay-queue&quot;, message_queue: &quot;my-dawdle-message-queue.fifo&quot; These values can also be set by using the environment variables DAWDLE_SQS_REGION, DAWDLE_SQS_DELAY_QUEUE, and DAWDLE_SQS_MESSAGE_QUEUE. The configuration should be managed either via config.exs or by setting the environment variables. Trying to mix the two will result in some changes being overwritten in surprising ways. Caveat emptor."},{"ref":"readme.html#setting-up-your-sqs-queues","title":"Dawdle - Setting Up Your SQS Queues","type":"extras","doc":"Obviously the configured SQS queues need to exist and be accessible by your application. AWS authentication is handled by ex_aws. If you&#39;re already using ex_aws for something else, your configuration should already be good. If not, follow the configuration instructions on that page to set up your AWS key. Dawdle uses two queues: one for normal messages and one for delayed events. The message queue must be a FIFO Queue and the delay queue must be a Standard Queue. They can be configured with default values, except that Receive Message Wait Time should be set to 20 seconds to enable long polling. The queues can be created using the aws CLI or from the AWS Control Panel. Here are example commands for creating the queues from the CLI: $ aws sqs create-queue --queue-name my-dawdle-delay-queue --attributes ReceiveMessageWaitTimeSeconds=20 { &quot;QueueUrl&quot;: &quot;https://xx-xxxx-x.queue.amazonaws.com/XXXXXXXXXXXX/my-dawdle-delay-queue&quot; } $ aws sqs create-queue --queue-name my-dawdle-message-queue.fifo --attributes FifoQueue=true,ReceiveMessageWaitTimeSeconds=20 { &quot;QueueUrl&quot;: &quot;https://xx-xxxx-x.queue.amazonaws.com/XXXXXXXXXXXX/my-dawdle-message-queue.fifo&quot; }"},{"ref":"readme.html#use","title":"Dawdle - Use","type":"extras","doc":"Full docs can be found at https://hexdocs.pm/dawdle. Event handlers are where Dawdle really begins to shine. An event is essentially just an Elixir struct. Define an event and event handler, then when you signal that event using Dawdle, the event handler will be called to process the event. Define the event defmodule MyApp.TestEvent do defstruct :foo, :bar end Create an event handler defmodule MyApp.TestEventHandler do use Dawdle.Handler, only: [MyApp.TestEvent] alias MyApp.TestEvent def handle_event(%TestEvent{} = event) do IO.puts(&quot;Handling event \#{inspect(event)}&quot;) :ok end end Signal an event t = %MyApp.TestEvent{foo: 1, bar: 2} Dawdle.signal(t) The event handler will execute on a node running the Dawdle application with pollers enabled. Note that if you are handling events on nodes different from where they are signaled, then you need to ensure that the event definintions are available in both places. Experimental API There is a basic, experimental API which involves passing a simple function to Dawdle.call/1. The function will execute on a node running the Dawdle application with pollers enabled. iex&gt; Dawdle.call(fn -&gt; IO.puts(&quot;Hello World!&quot;) end) :ok Hello World! Passing a function to Dawdle.call_after/2 will result in that function being called after the specified delay. iex&gt; Dawdle.call_after(2000, fn -&gt; IO.puts(&quot;Hello Future!&quot;) end) :ok # 2 seconds later Hello Future! This API is included for feedback and may be discontinued or extracted into a separate library for Dawdle 1.0."},{"ref":"readme.html#0-4-x-api-deprecated","title":"Dawdle - 0.4.x API (DEPRECATED)","type":"extras","doc":"Create a callback function iex&gt; callback = fn message -&gt; IO.inspect &quot;Received \#{message}&quot; end #Function&lt;6.99386804/1 in :erl_eval.expr/5&gt; Send your message iex(3)&gt; Dawdle.call_after(callback, &quot;Hello future&quot;, 2000) :ok # 2 seconds later &quot;Received Hello future&quot; This API will be removed before Dawdle 1.0."},{"ref":"readme.html#node-loss-tolerance","title":"Dawdle - Node Loss Tolerance","type":"extras","doc":"Because the events are managed outside of your BEAM VM(s), they will be preserved and handled by an available node even if the node that originally signaled them no longer exists."},{"ref":"readme.html#limits-and-caveats","title":"Dawdle - Limits and Caveats","type":"extras","doc":"SQS standard queues are not millisecond-precision timing devices. Their maximum delay precision is 1 second, so any timeouts given in fractions of a second will be rounded down. SQS standard queues guarantee at least once delivery. In practice it&#39;s almost always exactly once, but your code needs to handle the possibility that a given delayed function call will execute multiple times. SQS does not guarantee ordering on its standard queues, so if you set two delayed function calls with the same duration in quick succession, it&#39;s not guaranteed they&#39;ll execute in the same order they were set. SQS has an upper message size limit of 256KB, and the terms sent via it are Base64 encoded, so avoid sending large structures in your message. If you need a large bit of data as part of your message, stash it in a persistent store first and send the key through Dawdle. SQS delays are limited to 15 minutes. We handle longer delays by using multiple chained messages, so factor this into any capacity calculations you&#39;re doing."}]